import gtk
import gtk.gdk
from gazpacho.loader.loader import ObjectBuilder
import os.path
import sys

import alice

class RecolorWindow(object):

    def __init__(self, state, mw):

        self.mw = mw
        
        ob = ObjectBuilder(os.path.join(os.path.dirname(sys.argv[0]), 'recolor.glade'))

        self.win = ob.get_widget('recolorwindow')
        self.images = ob.get_widget('images')
        self.hueoffset = ob.get_widget('hueoffset')
        self.satoffset = ob.get_widget('satoffset')
        self.satmultiplier = ob.get_widget('satmultiplier')
        self.lumoffset = ob.get_widget('lumoffset')
        self.lummultiplier = ob.get_widget('lummultiplier')

        self.satoffset.set_increments(0.01, .1)
        self.satmultiplier.set_increments(0.01, .1)
        self.lumoffset.set_increments(0.01, .1)
        self.lummultiplier.set_increments(0.01, .1)

        cell = gtk.CellRendererText()
        self.images.append_column(gtk.TreeViewColumn('Prio', cell, text=4))
        self.images.append_column(gtk.TreeViewColumn('Category', cell, text=1))
        self.images.append_column(gtk.TreeViewColumn('Layer', cell, text=2))
        self.images.append_column(gtk.TreeViewColumn('Image', cell, text=3))

        self.images.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
        
        self.ignore_changes = False

        ob.signal_autoconnect(self)
        self.images.get_selection().connect("changed", self.images_changed)
        self.win.connect("delete-event", self.delete)

    def init(self, state):

        self.state = state

        self.image_model = gtk.ListStore(int, str, str, str, str)

        d = alice.dolls.dolls[state.doll]

        model = [ ]

        for cat, layers in d.layers.iteritems():
            for layer, l in layers.iteritems():
                for i in l.images:
                    model.append((i.priority, cat, layer, i.name, "%+d" % i.priority))

        model.sort()

        for i in model:
            self.image_model.append(i)

        self.images.set_model(self.image_model)

        self.recolor = alice.state.Recolor()
        self.selected = [ ]


    def images_changed(self, ts):

        model, selected = ts.get_selected_rows()

        self.recolor = alice.state.Recolor()
        self.selected = [ ]

        if not selected:
            return

        rc = None

        for i in selected:
            row = model[i]
            name = (row[1], row[2], row[3])

            self.selected.append(name)

            if name in self.state.recolors:
                rc = self.state.recolors[name]

        if not rc:
            rc = self.recolor

        try:

            self.ignore_changes = True

            self.hueoffset.set_value(rc.h_offset * 360.0)
            self.satoffset.set_value(rc.s_offset)
            self.satmultiplier.set_value(rc.s_multiply)
            self.lumoffset.set_value(rc.l_offset)
            self.lummultiplier.set_value(rc.l_multiply)

        finally:
            self.ignore_changes = False

    def color_changed(self, w):

        if self.ignore_changes:
            return True

        rc = alice.state.Recolor(
            h_offset = self.hueoffset.get_value() / 360.0,
            s_offset = self.satoffset.get_value(),
            s_multiply = self.satmultiplier.get_value(),
            l_offset = self.lumoffset.get_value(),
            l_multiply = self.lummultiplier.get_value(),
            )
        
        for i in self.selected:
            self.state.recolors[i] = rc

        self.mw.dirty()
        self.mw.render()


    def delete(self, w, e):
        self.win.hide()
        return True

    def show(self):
        self.win.present()
        
